/*
 * chvt.c - aeb - 940227 - Change virtual terminal
 */
#include <sys/types.h>
#include <sys/ioctl.h>
#include <linux/vt.h>
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <getopt.h>
#include <unistd.h>
#include "getfd.h"
#include "nls.h"
#include "version.h"

#define USER_WAIT_SLEEP_US		100000
#define USER_WAIT_MAX_ITERATIONS	50

static int fd;

static void chvt(int num)
{
    if (ioctl(fd,VT_ACTIVATE,num)) {
	perror("VT_ACTIVATE");
	exit(1);
    }
}

static int fgconsole(void)
{
    struct vt_stat vtstat;
    if (ioctl(fd, VT_GETSTATE, &vtstat)) {
	perror("VT_GETSTATE");
	exit(1);
    }
    return vtstat.v_active;
}

int
main(int argc, char *argv[]) {
    int c, num;
    int user_wait = 0;
    const struct option long_opts[] = {
	{ "version",	no_argument, NULL, 'V' },
	{ "userwait",	no_argument, NULL, 'u' },
    };

    set_progname(argv[0]);

    setlocale(LC_ALL, "");
    bindtextdomain(PACKAGE_NAME, LOCALEDIR);
    textdomain(PACKAGE_NAME);

    while ((c = getopt_long(argc, argv, "Vu", long_opts, NULL)) != -1) {
	switch (c) {
	    case 'V':
		print_version_and_exit();
	    case 'u':
		user_wait = 1;
		break;
	}
    }

    if (optind >= argc) {
	fprintf(stderr, _("usage: chvt [--userwait] N\n"));
	exit(1);
    }

    fd = getfd(NULL);
    num = atoi(argv[optind++]);
    chvt(num);

    if (user_wait) {
    	int active = 0;
	int i;
	for (i = 0; i < USER_WAIT_MAX_ITERATIONS; i++) {
	    if (fgconsole() == num) {
	    	active = 1;
	    	break;
	    }

	    chvt(num);
	    usleep(USER_WAIT_SLEEP_US);
	}
	if (!active) {
	    fprintf(stderr, _("VT change timed out\n"));
	    exit(1);
	}
    } else {
	if (ioctl(fd,VT_WAITACTIVE,num)) {
	    perror("VT_WAITACTIVE");
	    exit(1);
	}
    }
    exit(0);
}
